import random
import re
from io import StringIO
from datetime import datetime
#network
import requests
import ssl
import urllib.request
from urllib.request import Request, urlopen
#html processing
from html.parser import HTMLParser
from urlextract import URLExtract
#shorteners
import pyshorteners
# import feedparser
from newspaper import Article, Source, Config

#global variables
ua = "Mozilla Firefox Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:53.0) Gecko/20100101 Firefox/53.0"
max_len = 210 #280 max

########################################## GENERAL: ##########################################
def get_item(lis):
    items = lis.split(' ')
    if(len(items)==1):
        return str(items[0])
    else:
        return str(random.choice(items))

def timestamp():
    now = datetime.now()
    return now.strftime("%d/%m/%Y %H:%M:%S")

def datestamp():
    now = datetime.now()
    return now.strftime("%A, %B %d, %Y")

########################################## TEXT CLEANING: ####################################
class MLStripper(HTMLParser):
    def __init__(self):
        super().__init__()
        self.reset()
        self.strict = False
        self.convert_charrefs= True
        self.text = StringIO()
    def handle_data(self, d):
        self.text.write(d)
    def get_data(self):
        return self.text.getvalue()

def strip_tags(html):
    s = MLStripper()
    s.feed(html)
    text = s.get_data()
    return text

#strip unicode escape codes and sub with '
def strip_codes(str):
    return re.sub(r'\[?&#\d{4};\]?','\'',str)

def clean_text(str):
    #remove codes
    str = strip_codes(str)
    #remove any HTML tags
    str = strip_tags(str)
    #remove non-alphanumeric characters
    str = re.sub(r'\n',' ',str)
    str = re.sub(r'[^a-zA-Z0-9-_#@%&!.;:*$,|’\'\"\-() ]',' ', str)
    str = re.sub(r'Enlarge this image','',str)
    str = re.sub(r'toggle caption','',str)
    str = re.sub(r'BACKGROUND','',str)
    return str

def summarize_article(url,sent=3):
    print("Summarizing article at url:"+url)
    # print(url)
    config = Config()
    # config.MAX_SUMMARY=500
    config.MAX_SUMMARY_SENT=sent
    article = Article(url=url,config=config)
    article.download()
    article.parse()
    article.nlp()
    return {
            'title':clean_text(article.title),
            'summary':clean_text(article.summary),
            'url':url,
            'keywords':"#"+" #".join(article.keywords[0:3]),
            'img':article.top_image,
            'author':", ".join(article.authors),
            'text':article.text,
            'movies':article.movies
            }

def summarize_post(post):
    if('summary' in post): 
        post['summary'] = clean_text(post['summary'])
    elif('description' in post):
        post['summary'] = clean_text(post['description'])
    elif('abstract' in post):
        post['summary'] = clean_text(post['abstract'])
    try:
        parsed = summarize_article(get_url(post))
        if('author' not in post): post['author'] = parsed['author']
        post['img'] = parsed['img']
        post['movies'] = parsed['movies']
        if('summary' not in post): post['summary'] = parsed['summary']
    except Exception as e:
        print(e)
        post['img'] = ""
        post['author'] = ""
        post['movies'] = ""
    return post

def format_tweet(ti,url,summary="",intro="",bebukey=None):
    title = intro+clean_text(ti)
    title_len = len(title)
    max_url = max_len - title_len #shorten url based on title len
    url = shorten_link(url,max_url,bebukey)
    url_len = len(url)
    max_title = max_len - url_len-6
    truncated=''
    if(len(title)>max_title):
        truncated='...'
    title = title[0:max_title]
    title_len = len(title)
    summary = clean_text(summary)
    return {
        'tweet':"""{}{} {}""".format(title,truncated,url),
        'title':title,
        'summary':summary,
        'url':url
    }
########################################## URLS: ##########################################
def get_url(post):
    if('url' in post): return clean_url(post['url'])
    elif('full_url' in post): return clean_url(post['full_url'])
    elif('link' in post): return clean_url(post['link'])

def clean_url(url):
    try:
        return unshorten(url)
    except Exception as e:
        print(e)
        print("Error cleaning url: "+str(url))
        if('404' in str(e)):
            print("404 - not found.")
            raise Exception("404")
        elif('403' in str(e)):
            print("403 - blocked from visiting, but url may still be OK.")
    return url

#unshorten by visiting URL and returning url of opened site
def unshorten(url):
    req = Request(url, headers={'User-Agent': ua})
    url= urllib.request.urlopen(req,context=ssl._create_unverified_context()).url
    return url

def bebube(link,apikey):
    params={"longDynamicLink": "https://bebu.be/?link="+link, "suffix": { "option": "SHORT" } }
    url = """https://firebasedynamiclinks.googleapis.com/v1/shortLinks?key={}""".format(apikey)
    r = requests.post(url, json=params).json()['shortLink']
    return(r)

def shorten_link(url="",max_len=100,bebukey=None):
    # print("Shortening link: "+url)
    url = clean_url(url)
    s = pyshorteners.Shortener()
    if(len(url)< max_len):
        return url
    try:
        print("Getting bebu.be short link")
        short = bebube(url,bebukey)
        print(short)
        return short
    except:
        print("Getting is.gd link")
        short = s.isgd.short(url)
        return short

def follow_link(url,term = "udemy.com",level = 0,max_levels = 2):
    print("Checking url for term: "+url)
    if(term not in url and levels < max_levels):
        request_result=requests.get(url)
        follow_link(request_result,levels+1)
    else:
        return url

def get_hrefs(url):
    request_result=requests.get( url )
    soup = bs4.BeautifulSoup(request_result.text, "html.parser")
    raw_links = soup.find_all("a", href=True)
    return raw_links

def get_links(url):
    extractor = URLExtract()
    req = requests.get(url)
    page = req.text
    urls = extractor.find_urls(page)
    return urls

########################################## Tests: ##########################################
def test():
    text="Resumption Recommendation Expected - <p><span><span>The <em>Washington Post</em> reports that US officials are expected to give the go-ahead to resume using Johnson & Johnson's !!! #awesome!<span>SARS</span>-<span>CoV</span>-2 vaccine.</spa"
    print(clean_text(text))

if(__name__=='__main__'):
    test()


import requests
import bs4
import praw
import random
import urllib
import urllib.request
import re
# import nltk
import json
from func_timeout import func_timeout, FunctionTimedOut , func_set_timeout
from newspaper import Article, Source, Config

# from .. import utils
try:
    from .. import utils
except:
    from synchron import utils

@func_set_timeout(30)
def check_reddit_post(post):
    rejects = ['reddit.com','redd.it','reddit','nsfw','redd','red']
    print("Checking post: %s"%(post.title))
    if any(term in post.url for term in rejects):
        print("Invalid Post URL")
        return False
    elif any(term in post.title.lower() for term in rejects):
        print("Invalid Post name")
        return False
    else:
        post.title = utils.clean_text(post.title)
        print("Verifying and cleaning URL for post: "+post.url)
        try:
            post.url= utils.clean_url(post.url)
        except Exception as e:
            print(e)
            print("Invalid link. Rejecting post")
            return False
        print("Going with reddit post: %s"%(post.title))
        return True

def format_post(rp,bebukey):
    post = {}
    print("Formatting post")
    tweet = utils.format_tweet(ti=rp.title,url=rp.url,summary="",intro="",bebukey=bebukey)
    summary = utils.summarize_article(rp.url)
    post['tweet']=tweet['tweet']
    post['title'] = rp.title
    post['description'] = summary['summary']
    post['author'] = summary['author']
    post['url'] = tweet['url']
    post['img'] = summary['img']
    post['tags'] = summary['keywords']
    # post['text'] = summary['text']
    # print(pos
    # t)
    return post

def get_reddit_links(ci, cs, ua="Mozilla",subreddit="science",max_attempts=5,count=1,bebukey=None):
    # print(bebukey)
    reddit = praw.Reddit(client_id=ci,client_secret=cs,user_agent=ua)
    #keep trying posts until a good post is obtained
    attempts = 0
    post = False
    while(not post and attempts < max_attempts):
        attempts += 1
        #select subreddit from specified input for each attempt
        sr = utils.clean_text(utils.get_item(subreddit)).lower()
        print("Getting post from subreddit %s. Attempt %d"%(sr, attempts))
        #pick link, but only if not a Reddit link:
        try:
            links = reddit.subreddit(sr).hot(limit=100)
        except Exception as e:
            print(e)
            print("Problem retrieving subreddit %s"%(subreddit))
        #get high-scoring links... really hot
        hot_links = [l for l in links if l.score > 100]
        if(len(hot_links)<count): 
            raise Exception("Too few links in subreddit: %s"%(sr))
        if(count == 1): #case for selecting 1 link
            for i in range(max_attempts):
                post = random.choice(hot_links)
                try:
                    if(check_reddit_post(post)): 
                        try:
                            print("Formatting post")
                            # formatted = utils.format_tweet(ti=post.title,url=post.url,summary="",intro="",bebukey=bebukey)
                            formatted = format_post(post,bebukey)
                            print(formatted['description'])
                            return formatted
                        except FunctionTimedOut:
                            print("Timed out. Rejecting post.")
                        except Exception as e:
                            print(e)
                            print("Post was invalid. Selecting another.")
                except:
                    print("Rejected post.")
        else:
            posts = []
            bads = 0
            while(len(posts)<count):
                post = hot_links.pop(0)
                try:
                    if(check_reddit_post(post)): 
                        try:
                            # formatted = utils.format_tweet(ti=post.title,url=post.url,summary="",intro="",bebukey=bebukey)
                            formatted = format_post(post,bebukey)
                            print(formatted['description'])
                            posts.append(formatted)
                        except FunctionTimedOut:
                            print("Timed out. Rejecting post.")
                        except Exception as e:
                            print(e)
                            print("Post was invalid. Selecting another.")
                    else:
                        bads+=1
                        print("Bad results count: "+str(bads))
                except:
                    bads+=1
                    print("Rejected post")

            return posts


############################# ACCESSORS #############################
def get_update(client_id = "", client_secret = "", user_agent="Mozilla",subreddit="technology",max_attempts=5,bebukey=None):
    print("Getting Reddit post.")
    post = get_reddit_links(client_id,client_secret,user_agent,subreddit,max_attempts,1,bebukey)
    if(not post): raise Exception("No post found!")
    return post

def get_multiple(client_id = "", client_secret = "", user_agent="Mozilla",subreddit="technology",count=5,bebukey=None):
    print("Getting Reddit post.")
    posts = get_reddit_links(client_id,client_secret,user_agent,subreddit,5,count,bebukey)
    if(not posts): raise Exception("No posts found!")
    return posts

#get_posts = the main function. 
#count is required
#additional parameters can be in kwargs
def get_posts(count=1,**kwargs):
    if('subreddit' in kwargs):
        subreddit = kwargs['subreddit']
    else:
        subreddit = "technology"

    if('bebukey' in kwargs):
        bebukey = kwargs['bebukey']
    else: 
        bebukey = None 

    try:
        client_id = kwargs['REDDIT_CLIENT_ID']
        client_secret = kwargs['REDDIT_CLIENT_SECRET']
    except:
        client_id = kwargs['client_id']
        client_secret = kwargs['client_secret']

    posts = get_reddit_links(
        client_id,
        client_secret,
        utils.ua,
        subreddit,
        5,
        count ,
        bebukey)
    if(not posts): raise Exception("No posts found!")
    return posts

if __name__ == "__main__":
    # import env
    with open("env.json",'r') as f:
        env = json.load(f)
    SUBREDDIT= 'science technology cybersecurity'
    user_agent="Python"
    # print(get_update(env.client_id,env.client_secret,env.user_agent,SUBREDDIT))
    posts = get_multiple(env['REDDIT_CLIENT_ID'],env['REDDIT_CLIENT_SECRET'],'useragent',"technology",5,bebukey=env['BEBUKEY'])
    print("Posts: %d"%(len(posts)))
    print(posts)

# def summarize_article(url):
#     config = Config()
#     # config.MAX_SUMMARY=500
#     config.MAX_SUMMARY_SENT=3
#     article = Article(url=url,config=config)
#     article.download()
#     article.parse()
#     article.nlp()
#     return {
#             'title':article.title,
#             'summary':utils.clean_text(article.summary),
#             'url':url,
#             'keywords':"#"+" #".join(article.keywords[0:3]),
#             'img':article.top_image,
#             'author':", ".join(article.authors),
#             'text':article.text,
#             'movies':article.movies
#             }


import requests
import bs4
import praw
import random
import urllib.request
import re
import nltk
import pyshorteners
import json
from urlextract import URLExtract
import urllib.request
from urllib.request import Request, urlopen
from func_timeout import func_timeout, FunctionTimedOut , func_set_timeout

default_ua = "Mozilla Firefox Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:53.0) Gecko/20100101 Firefox/53.0"
max_len = 240
try:
    from .. import utils
except:
    from synchron import utils

def get_udemy_info(url):
    max_len = 260
    #test
    req = Request(url, headers={'User-Agent': default_ua})
    res= urllib.request.urlopen(req)
    reqs = requests.get(url)
    soup = bs4.BeautifulSoup(res.read(), 'html.parser')
    course_title = soup.find(class_='clp-lead__title').text
    course_title = utils.clean_text(course_title)
    course_subtitle = soup.find(class_='clp-lead__headline').text
    course_subtitle = utils.clean_text(course_subtitle)
    language = ""
    course_language = soup.find(class_='clp-lead__locale').text.lower().strip()
    img_src = ""
    images = soup.findAll("img")
    for img in images:
        if("course" in img['src']):
            if('?' in img['src']): img_src = img['src'].split('?')[0]
    return {
        "title":course_title,
        "subtitle":course_subtitle,
        "language":course_language,
        "img":img_src
    }

#Grab Udemy course info from link
def get_udemy_link(url):
    extractor = URLExtract()
    req = requests.get(url)
    page = req.text
    # urls = extractor.find_urls(str(soup))
    urls = extractor.find_urls(page)
    for url in urls:
        if('udemy.com' in url):
            print("Extracting info from link: "+str(url))
            try:
                coupon = url.split('=')[1]
            except:
                coupon = None
            try:
                info = get_udemy_info(url)
                return {
                    "title":info['title'],
                    "subtitle":info['subtitle'],
                    "url":url,
                    "coupon":coupon,
                    "language":info['language'],
                    "img":info['img']
                }  
            except Exception as e:
                print(e)
                print("Unable to extract info from url: "+str(url))

@func_set_timeout(10)
def get_udemy_post(red,bebukey=None):
    max_len = 240
    utags = "#udemyfreebies #udemycoupon #udemy #udemyfree #udemysale #udemyflashsale"
    url = red.url
    udemy = get_udemy_link(url)
    title = udemy['title']#+" "+udemy['subtitle']
    title_len = len(title)
    max_url = max_len-title_len #shorten url based on title len
    try:
        url = utils.shorten_link(udemy['url'],max_url,bebukey)
    except:
        print("Unable to shorten url")
    url_len = len(url)
    max_title = max_len - url_len
    truncated=''
    if(len(title)>max_title):
        truncated='...'
        title = title[0:max_title]
    title_len = len(title)
    if(udemy['coupon']):
        coupon_text="FREE COUPON CODE: "+udemy['coupon']
    else:
        coupon_text = ""
    post = """🔥SALE!🔥 
{} 
{} 
{} {}""".format(title,coupon_text,udemy['url'],utags)
    post_2="""{} 
{}...
{} 
{}""".format(title,udemy['subtitle'],coupon_text,udemy['url'])
    udemy['tweet']=post[0:278]
    udemy['tweet_2']=post_2
    return udemy

def get_reddit_link(ci, cs, subreddit="udemyfreebies"):
    reddit = praw.Reddit(client_id=ci,client_secret=cs, user_agent=default_ua)
    links = reddit.subreddit(subreddit).hot(limit=100)
    links = [l for l in links]
    pick_link = random.choice(links)
    return(pick_link)

def get_reddit_all(ci,cs,subreddit="udemyfreebies"):
    reddit = praw.Reddit(client_id=ci,client_secret=cs, user_agent=default_ua)
    links = reddit.subreddit(subreddit).hot(limit=100)
    links = [l for l in links]
    # pick_link = random.choice(links)
    return(links)

def get_update(client_id, client_secret,subreddit="udemyfreebies",bebukey=None):
    max_tries = 5
    attempt = 0
    if(len(subreddit)==0): subreddit = "udemyfreebies"
    while(attempt<max_tries):
        attempt+=1
        try:
            red = get_reddit_link(client_id,client_secret,subreddit)
            post = get_udemy_post(red,bebukey)
            if('english' in post['language'].lower()):
                return post
                break
            else:
                print("Post isn't English. Skipping.")
        except FunctionTimedOut:
                print("Timed out. Rejecting post.")
        except Exception as e:
            print(e)
            print("Encountered problem getting link. Attempt: "+str(attempt))

def get_multiple(client_id, client_secret, post_number=5,bebukey=None):
    posts = get_reddit_all(client_id,client_secret)
    tweets = []
    for red in posts:
        if(len(tweets)<post_number):
            try:
                print("Post: %s"%(red.title))
                post = get_udemy_post(red,bebukey)
                if('english' in post['language']):
                    tweets.append(post)
                else:
                    print("Post isn't English. Skipping.")
            except FunctionTimedOut:
                print("Timed out. Rejecting post.")
            except Exception as e:
                print(e)
                print("Unable to add post: %s"%(red.title))
        else:
            break
    return tweets

#get_posts = the main function. 
#count is required
#additional parameters can be in kwargs
def get_posts(count=1,**kwargs):
    if('bebukey' in kwargs):
        bebukey = kwargs['bebukey']
    else: 
        bebukey = None 

    try:
        client_id = kwargs['REDDIT_CLIENT_ID']
        client_secret = kwargs['REDDIT_CLIENT_SECRET']
    except:
        client_id = kwargs['client_id']
        client_secret = kwargs['client_secret']

    posts = get_multiple(
        client_id,
        client_secret,
        count,
        bebukey)
    if(not posts): raise Exception("No posts found!")
    return posts

def test():
    with open('page.html', 'r') as f:
        contents = f.read()
        soup = bs4.BeautifulSoup(contents, 'html.parser')
        # print(soup)
        # print("udemy.com" in contents)
        # print(soup.findall('form'))
        # for el in soup.find_all('form'):
        #     # print(el)
        #     if("udemy" in el):
        #         print(el)
        extractor = URLExtract()
        urls = extractor.find_urls(str(soup))
        for url in urls:
            if('udemy.com' in url):
                print(url)
                coupon = url.split('=')[1]
                print("coupon: %s"%(coupon))

if __name__ == "__main__":
    with open("env.json",'r') as f:
        env = json.load(f)
    # posts = get_multiple(env['REDDIT_CLIENT_ID'],env['REDDIT_CLIENT_SECRET'],'useragent',"technology",5,bebukey=env['BEBUKEY'])
    print(get_update(env['REDDIT_CLIENT_ID'],env['REDDIT_CLIENT_SECRET'],bebukey=env['BEBUKEY']))
    # posts = get_multiple(env.client_id,env.client_secret,2)
    # print("Posts: %d"%(len(posts)))
    # print(posts)



import setuptools.command.build_py
from setuptools import setup, find_packages
from setuptools.command.install import install as _install

#custom post-installation steps go here:
class Install(_install):
    def run(self):
        _install.do_egg_install(self)
        #nothing else to do

setup(
    cmdclass={
        'install': Install,
    },
    name='SynChron',
    description='SynChron',
    url='https://github.com/sequencecentral/SynChron.git',
    # git+https://github.com/sequencecentral/twitter-bot.git@main#egg=twitterbot
    author='Steve Ayers, Ph.D.',
    author_email='steve@sequenccecentral.com',
    # install_requires=[],
    version='1.2.5',
    license='MIT',
    # packages=['synchronicity','synchronicity.quotewidget'],
    packages = find_packages(),
    include_package_data = True,
    package_data={'': ['config.json','sources.json']},
    # Needed to actually package something
    # Needed for dependencies
    # install_requires=[''],
    # *strongly* suggested for sharing
    long_description="Synchronicity package", #open('README.md').read(),
    #Don't install Pyrebase. It ruins everything
    install_requires=['appdirs==1.4.4', 'beautifulsoup4==4.9.3', 'breadability==0.1.20', 'bs4==0.0.1', 'certifi==2020.12.5', 'chardet==4.0.0', 'click==7.1.2', 'cssselect==1.1.0', 'docopt==0.6.2', 'feedfinder2==0.0.4', 'feedparser==6.0.2', 'filelock==3.0.12', 'func-timeout==4.3.5', 'gcloud==0.17.0', 'googleapis-common-protos==1.53.0', 'httplib2==0.19.1', 'idna==2.10', 'install==1.3.4', 'jieba3k==0.35.1', 'joblib==1.0.1', 'jws==0.1.3', 'lxml==4.6.3', 'newspaper3k==0.2.8', 'nltk==3.6.1', 'oauth2client==3.0.0', 'oauthlib==3.1.0', 'Pillow==8.2.0', 'praw==7.2.0', 'prawcore==2.0.0', 'protobuf==3.16.0rc1', 'pyasn1==0.4.8', 'pyasn1-modules==0.2.8', 'pycountry==20.7.3', 'pycryptodome==3.4.3', 'pyparsing==2.4.7', 'pyshorteners==1.0.1', 'PySocks==1.7.1', 'python-dateutil==2.8.1', 'python-jwt==2.0.1', 'PyYAML==5.4.1', 'regex==2021.4.4', 'requests>=2.0', 'requests-file>=1.5', 'requests-oauthlib>=1.3', 'requests-toolbelt>=0.7', 'rsa==4.7.2', 'sgmllib3k==1.0.0', 'six==1.15.0', 'soupsieve==2.2.1', 'sumy==0.8.1', 'tinysegmenter==0.3', 'tldextract==3.1.0', 'tqdm==4.60.0', 'tweepy==3.10.0', 'update-checker==0.18.0', 'uritools==3.0.1', 'urlextract==1.2.0', 'urllib3==1.26.4', 'websocket-client==0.58.0']
    # install_requires=open('requirements.txt').read(), #['click==7.1.2','joblib==1.0.1','nltk==3.6.1','regex==2021.4.4','tqdm==4.60.0'],
    # We will also need a readme eventually (there will be a warning)
    # long_description=open('README.txt').read(),
)

#to make an egg:
#python setup.py bdist_egg
#egg-info added